/*
 * Copyright 2010-2013 Intel Corporation.
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, version 2.1.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * Disclaimer: The codes contained in these modules may be specific
 * to the Intel Software Development Platform codenamed Knights Ferry,
 * and the Intel product codenamed Knights Corner, and are not backward
 * compatible with other Intel products. Additionally, Intel will NOT
 * support the codes or instruction set in future products.
 *
 * Intel offers no warranty of any kind regarding the code. This code is
 * licensed on an "AS IS" basis and Intel is not obligated to provide
 * any support, assistance, installation, training, or other services
 * of any kind. Intel is also not obligated to provide any updates,
 * enhancements or extensions. Intel specifically disclaims any warranty
 * of merchantability, non-infringement, fitness for any particular
 * purpose, and any other warranty.
 *
 * Further, Intel disclaims all liability of any kind, including but
 * not limited to liability for infringement of any proprietary rights,
 * relating to the use of the code, even if Intel is notified of the
 * possibility of such liability. Except as expressly stated in an Intel
 * license agreement provided with this code and agreed upon with Intel,
 * no license, express or implied, by estoppel or otherwise, to any
 * intellectual property rights is granted herein.
 */

#ifndef __MECHANISM_PINNED_BUFFER_H__
#define __MECHANISM_PINNED_BUFFER_H__

#include "buffer.h"

class COIBufferPinnedBase : public COIBuffer
{
public:
    COIBufferPinnedBase(  uint64_t      size,
                          uint32_t      num_procs,
                    const COIPROCESS*   procs,
                    uint8_t             m_page_type);
    virtual ~COIBufferPinnedBase();

    virtual uint64_t RequiredSize();

    virtual COIRESULT MoveTo(    COIPROCESS          move_to,
                            COI_ACCESS_FLAGS    flags,
                            uint64_t            offset,
                            uint64_t            length,
                            COIEVENT            in_event,
                            RemapList&          in_remap_list,
                            COIEVENT&           out_complete,
                            COIEVENT&           out_refcount,
                            _COIRunFunction*   run_Function_node);

    virtual COIRESULT Map(      uint64_t        offset,
                                uint64_t        len,
                                COI_MAP_TYPE    type,
                                uint32_t        num_deps,
                          const COIEVENT*       deps,
                                COIEVENT*       out_complete,
                                COIMAPINSTANCE* out_instance,
                                void**          out_data);
    bool Mapped();
};

class COIBufferPinned : public COIBufferPinnedBase
{
public:
    typedef pair<physical_region*,bool> elem_t;
    typedef list<elem_t>                physical_list_t;

    COIBufferPinned(      uint64_t      size,
                          uint32_t      num_procs,
                    const COIPROCESS*   procs,
                    uint8_t             m_page_type);
    virtual ~COIBufferPinned();

    virtual COIRESULT MoveTo(    COIPROCESS          move_to,
                            COI_ACCESS_FLAGS    flags,
                            uint64_t            offset,
                            uint64_t            length,
                            COIEVENT            in_event,
                            RemapList&          in_remap_list,
                            COIEVENT&           out_complete,
                            COIEVENT&           out_refcount,
                            _COIRunFunction*   run_Function_node);

protected:
    void    cleanup();

    // Separate list from the base class because we ALWAYS own these blocks and
    // they are not stealable AND our shadow memory always points to them on
    // the remote side so we don't need to do any walking of blocks so we want
    // to keep the base class block list empty
    physical_list_t m_physical_blocks;

    bool            m_remapped;
};

class COIBufferPinnedFromMemory : public COIBufferPinnedBase
{
public:
    COIBufferPinnedFromMemory(        uint64_t      size,
                                      uint32_t      num_procs,
                                const COIPROCESS*   procs,
                                      void*         memory,
                                      uint8_t       m_page_type);
    virtual ~COIBufferPinnedFromMemory();

protected:
    void    cleanup();

};

class COIBufferPinnedMultipleProcs : public COIBufferPinnedFromMemory
{
public:
    COIBufferPinnedMultipleProcs(
                          uint64_t      size,
                          uint32_t      num_procs,
                    const COIPROCESS*   procs,
                    uint8_t             m_page_type);
    virtual ~COIBufferPinnedMultipleProcs();

protected:
    void cleanup();
};

#endif // __MECHANISM_PINNED_BUFFER_H__
