/*
 * Copyright 2010-2013 Intel Corporation.
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, version 2.1.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * Disclaimer: The codes contained in these modules may be specific
 * to the Intel Software Development Platform codenamed Knights Ferry,
 * and the Intel product codenamed Knights Corner, and are not backward
 * compatible with other Intel products. Additionally, Intel will NOT
 * support the codes or instruction set in future products.
 *
 * Intel offers no warranty of any kind regarding the code. This code is
 * licensed on an "AS IS" basis and Intel is not obligated to provide
 * any support, assistance, installation, training, or other services
 * of any kind. Intel is also not obligated to provide any updates,
 * enhancements or extensions. Intel specifically disclaims any warranty
 * of merchantability, non-infringement, fitness for any particular
 * purpose, and any other warranty.
 *
 * Further, Intel disclaims all liability of any kind, including but
 * not limited to liability for infringement of any proprietary rights,
 * relating to the use of the code, even if Intel is notified of the
 * possibility of such liability. Except as expressly stated in an Intel
 * license agreement provided with this code and agreed upon with Intel,
 * no license, express or implied, by estoppel or otherwise, to any
 * intellectual property rights is granted herein.
 */

#ifndef __MECHANISM_STREAMING_BUFFER_H__
#define __MECHANISM_STREAMING_BUFFER_H__

#include "buffer.h"

class _StreamingBuffer : public COIBufferBase
{
public:
    _StreamingBuffer(         uint64_t        size,
                              COI_BUFFER_TYPE type,
                              uint32_t        flags,
                        const void*           init_data,
                              uint32_t        num_procs,
                        const COIPROCESS*     procs,
                        const void*           donated_memory,
                        uint8_t               m_page_type);
    virtual ~_StreamingBuffer();

    virtual void*       SinkAddress(COIPROCESS p);
    virtual void*       LocalAddress();

    void MarkUsedInPipe();

    virtual bool Mapped() = 0;

    virtual _COIBuffer* CreateSubBuffer(    uint64_t length,
                                            uint64_t offset);

    virtual COIRESULT Map(        uint64_t          offset,
                                  uint64_t          len,
                                  COI_MAP_TYPE      type,
                                  uint32_t          num_deps,
                            const COIEVENT*         deps,
                                  COIEVENT*         out_complete,
                                  COIMAPINSTANCE*   out_instance,
                                  void**            out_data) = 0;

    // will make the buffer valid on the process move_to
    // will wait on the input deps
    // is done when out_complete fires
    virtual COIRESULT Move(       COIPROCESS        move_to,
                                  COI_ACCESS_FLAGS  flag,
                                  uint64_t          offset,
                                  uint64_t          length,
                                  uint32_t          num_deps,
                            const COIEVENT*         deps,
                                  RemapList&        in_remap_list,
                                  COIEVENT*         out_complete,
                                  COIEVENT*         out_refcount,
                                  _COIRunFunction* run_Function_node) = 0;    
    

    // Writes to the buffer from a local buffer at source
    virtual COIRESULT Write(const void*         source,
                            const COIPROCESS    target_process,
                                  uint64_t      offset,
                                  uint64_t      length,
                                  COI_COPY_TYPE type,
                                  uint32_t      num_deps,
                            const COIEVENT*     deps,
                                  COIEVENT*     out_complete,
                                  uint64_t      move_offset,
                                  uint64_t      move_length);

    // Reads from the buffer to a local buffer at dest
    virtual COIRESULT Read(       void*         dest,
                                  uint64_t      offset,
                                  uint64_t      length,
                                  COI_COPY_TYPE type,
                                  uint32_t      num_deps,
                            const COIEVENT*     deps,
                                  COIEVENT*     out_complete);

    virtual COIRESULT Copy( _COIBuffer*     src,
                            uint64_t        dst_offset,
                      const COIPROCESS      target_process,
                            uint64_t        src_offset,
                            uint64_t        length,
                            COI_COPY_TYPE   type,
                            uint32_t        num_deps,
                      const COIEVENT*       deps,
                            COIEVENT*       out_complete,
                            uint64_t      move_offset,
                            uint64_t      move_length);

    virtual COIRESULT WriteMultiD( 
                            const   struct arr_desc*  source,
                            const   COIPROCESS        target_process,
                                    uint64_t          offset,
                                    struct arr_desc*  dest,
                                    COI_COPY_TYPE     type,
                                    uint32_t          num_deps,
                            const   COIEVENT*         deps,
                                    COIEVENT*         out_complete,
                                    uint64_t          move_offset,
                                    uint64_t          move_length);

    virtual COIRESULT ReadMultiD(
                                    struct arr_desc*    dest,
                                    uint64_t            offset,
                                    struct arr_desc*    src,
                                    COI_COPY_TYPE       type,
                                    uint32_t            num_deps,
                              const COIEVENT*           deps,
                                    COIEVENT*           out_complete);

    virtual COIRESULT   BufferSetState(
                            COIPROCESS             in_Process,
                            COI_BUFFER_STATE       in_State,
                            COI_BUFFER_MOVE_FLAG   in_DataMove,
                            uint32_t                in_NumDependencies,
                    const   COIEVENT*               in_pDependencies,
                            COIEVENT*               out_pCompletion);

    virtual void  DeleteVersion(_COIBuffer *p);

    _COIBuffer* GetCurrentVersion();

    virtual COIRESULT Destroy() = 0;
    
    virtual COIRESULT MultiDMathCheck(
                      struct arr_desc *     src,
                      struct arr_desc *     dst,
                      uint64_t              off,
                      bool                  write);

protected:
    bool                    m_usedInPipe;
    _COIBuffer*             p_latestBuffer;
    uint64_t                m_refCount;

    //Parameters for creating a buffer
    uint32_t                m_flags;
    void                   *p_initData;
    uint32_t                m_numProcs;
    const COIPROCESS*       p_procs;
    bool                    m_resourceExhausted;
};

class MapStreamingInstanceImpl;

class _ToSourceBuf : public _StreamingBuffer
{
public:
    _ToSourceBuf(   uint64_t            size,
                    COI_BUFFER_TYPE     type,
                    uint32_t            flags,
                    const void*         init_data,
                    uint32_t            num_procs,
                    const COIPROCESS*   procs,
                    const void*         donated_memory,
                    uint8_t             m_page_type);
    virtual ~_ToSourceBuf();

    virtual COIRESULT Map(        uint64_t          offset,
                                  uint64_t          len,
                                  COI_MAP_TYPE      type,
                                  uint32_t          num_deps,
                            const COIEVENT*         deps,
                                  COIEVENT*         out_complete,
                                  COIMAPINSTANCE*   out_instance,
                                  void**            out_data);

    virtual COIRESULT Move(       COIPROCESS        move_to,
                                  COI_ACCESS_FLAGS  flag,
                                  uint64_t          offset,
                                  uint64_t          length,
                                  uint32_t          num_deps,
                            const COIEVENT*         deps,
                                  RemapList&        in_remap_list,
                                  COIEVENT*         out_complete,
                                  COIEVENT*         out_refcount,
                                  _COIRunFunction* run_Function_node);                            

                                 

    virtual COIRESULT Destroy();

    COIRESULT CreateVersion();
    COIRESULT Unmap(MapStreamingInstanceImpl* map);

    virtual bool Mapped ();

protected:
    std::list <_COIBuffer*>       m_versions; //Versions 
    list<MapInstance*>            m_mappings;
    bool                          m_RunFuncMove;
};

class _ToSinkBuf : public _StreamingBuffer
{
public:
    _ToSinkBuf(       uint64_t          size,
                      COI_BUFFER_TYPE   type,
                      uint32_t          flags,
                const void*             init_data,
                      uint32_t          num_procs,
                const COIPROCESS*       procs,
                const void*             donated_memory,
                uint8_t                 m_page_type);
    virtual ~_ToSinkBuf();

    virtual COIRESULT Map(        uint64_t          offset,
                                  uint64_t          len,
                                  COI_MAP_TYPE      type,
                                  uint32_t          num_deps,
                            const COIEVENT*         deps,
                                  COIEVENT*         out_complete,
                                  COIMAPINSTANCE*   out_instance,
                                  void**            out_data);

    virtual COIRESULT Move(       COIPROCESS        move_to,
                                  COI_ACCESS_FLAGS  flag,
                                  uint64_t          offset,
                                  uint64_t          length,
                                  uint32_t          num_deps,
                            const COIEVENT*         deps,
                                  RemapList&        in_remap_list,
                                  COIEVENT*         out_complete,
                                  COIEVENT*         out_refcount,
                                  _COIRunFunction* run_Function_node);                                                                    
                                 

    virtual void DeleteVersion(_COIBuffer *p);

    virtual COIRESULT Destroy();

    virtual bool Mapped ();

private:
    std::map <_COIBuffer*,uint64_t>                 m_versions; 
};

//DON'T create a instance of this class on stack. The Unmap functions calls
//delete on this
class MapStreamingInstanceImpl : public MapInstance
{
public:
    MapStreamingInstanceImpl(   _ToSourceBuf*       b,
                                uint64_t            offset,
                                uint64_t            size,
                                MapInstanceImpl*    mapInstance,
                                _COIBuffer*         version);
    ~MapStreamingInstanceImpl();

    virtual COIRESULT Unmap(      uint32_t  num_deps,
                            const COIEVENT* deps,
                                  COIEVENT* out_complete);

private:
    _ToSourceBuf*          m_buffer;
    uint64_t               m_offset, m_size;
    MapInstanceImpl*       m_mapping;
    _COIBuffer*            m_version;
};

#endif // __MECHANISM_STREAMING_BUFFER_H__
